

{  Global Turbo Pascal Unit Documentation :

           File name;  HPLJ.pas                  author Neil Everhart, 1988.

           This source file is the placed in the public domain by the author.
           Blanket permission to use/incorporate this unit into application
           software is granted with no warranty expressed or implied.

           This source file conforms to the Borland Turbo Pascal language
           definition(s) for Borland's versions 4.0 and 5.0 Turbo Pascal.
           The source will have to be compiled by the version that you are
           using.

           This unit has been used by the author in several applications.
           Each of these applications have had the HP Laser Jets, of one
           model or another, loacated on the LANs as shared printers.  The
           applications print to the logical local print device, the network
           software managed the printing redirection.

           This unit is written to facilitate modification for specific
           tasks with the intent that most changes can be accomplished by
           altering the values of the constants that are local to the unit.

           Using programmers, PLEASE rename the source (thus the "TPU") if
           you alter the original source for your application.

           Page orientation is in landscape mode with the 0,0 graphic
           coordinate located at the upper left corner of the landscape page.

           Please note that the paper travels through the laser printer in
           the 'page' orientation and the laser scan is left to right
           accross the page from top to bottom.  The unit manages the
           bit map so that the user need only worry where on the page to
           place the graphic lines/images.

           This engine allows printing of text (at 10 CPI by 6 LPI) to the
           HP Laser printer and simple graphics.  The concept is to form a
           bit map of the graphics in memory, on the heap, then process the
           bit map to the printer after the text has been passed to the
           printer.

           As indicated above, the intent of the unit is to provide simple
           graphics with text.  The functionallity provided allows drawing
           lines and/or boxes around text.  As such, the calls are oriented
           around character cell coordinates.


           Use the following sequence:

           1.  As a unit, the memory area is automatically allocated and
               initializied at start up by INIT.  The memory utilized by this
               unit is on the heap and consumes apx 55Kb.  To reinitialize or
               clear the bit map, call INITBITMAP before processing a second
               page (with differing graphics) to the printer.

           2.  Generate the bit map graphics using;

              2.1  SetBit( col, row, BitOnMode ) where

                   This procedure is used by the unit to set and unset all
                   dots/bits in the bit map.  It is included as a global
                   call for those that may want more control (added functions)
                   then supplied in the stock unit.

                   Also, please note that a global boolean, SetBitError is
                   provided to indicate when this procedure does not complete
                   as expected ( call was out of range ).

                       col  =  is an integer value within the range of
                               the graphics page (0 to 824) indicating
                               the x axis ordinal value.

                       row  =  is an integer value within the range of
                               the graphics page (0 to 636) indicating
                               the y axis ordinal value.

                BitOnMode  =   is a boolean flag; if true the bit is set
                               and if false, the bit is reset to "zero" if
                               it was already set on.

              2.2  DOLINE( X1, Y1,  X2, Y2, Width, Space )  where


                    X1, Y1  =  line origination coordinates in inches.
                               These arguments may be reals.

                    X2, Y2  =  line termination coordinates in inches.
                               These arguments may be reals.

                     Width  =  an integer value indicating the number
                               of (0.01333 inch) dot rows wide.

                               a negative Width value will cause a dased
                               line to be drawn.

                     Space  =  an integer value indicating the number
                               of (0.01333 inch wide) dot rows of white
                               space between double lines.  If Space is
                               a value of 0 then one line (of Width) is
                               drawn.  The added line width accumulates
                               to the right and/or downward from a single
                               line of the same coordinates.

                     DoLine is used to draw lines in the bit map and can
                     be called by the user.  The following procedures call
                     DoLine to accomplish their functions.

              2.3  HchrLn( StartX, StartY, EndX, Width, Space ) where

                     This procedure is "Horizontal Character Line" meaning
                     that the line drawn is from the center of the StartX/
                     StartY character cell to the center of the EndX char.
                     cell in the StartY row.

                     See DOLINE for "Width" and "Space" argument defintiton.


              2.4  VchrLn( StartX, StartY, EndY, Width, Space ) where

                     This procedure draws vertical lines from the StarX/StartY
                     to EndY center character cells.

                     See DOLINE for "Width" and "Space" argument defintiton.

              2.5  HbndLn( StartX, StartY, EndX, Width, Space ) where

                     This procedure is "Horizontal Boundry Line" meaning
                     that the line drawn is from the StartX/StartY character
                     cell to the EndX char. cell in the StartY row.  Note,
                     the line starts at the upper left edge of the cell and
                     ends at the upper left edge of the EndX cell.  Please
                     note that line ends at the left boundry  of the EndX
                     cell and does not extend over that cell.

                     See DOLINE for "Width" and "Space" argument defintiton.


              2.6  VbndLn( StartX, StartY, EndY, Width, Space ) where

                     This procedure draws vertical lines from the StarX/StartY
                     to EndY character cells on the left bounry of Y value cells.

                     See DOLINE for "Width" and "Space" argument defintiton.


              2.7  FrameChr( StartX, StartY, EndX, EndY ) and
                   FrameBnd( StartX, StartY, EndX, EndY ) where

                     These procedure draw boxes or frames around the selected
                     area.  The minimum size of the frame is one cell;  this
                     means that EndX and EndY values should be greater then
                     StartX and StartY values ( except as indicated below ).

                     The FrameBnd procedure addes one character cell to EndX
                     value so as to enclose the slected area.  It is possible
                     to draw a box around one character (or col) by using
                     a EndX and StartX of the same value.

                     Also, the boxes are drawn with single line width and no
                     double lines.  To add heavier lines and/or double lines,
                     call one of the line draw procedures and overdraw the
                     desired line.


           3.  Reset the printer by calling RESET;

           4.  Set printer to landscape mode by calling LANDSCAPEMODE.

           5.  Process the text to the printer, not exceeding line 60.
               Note if line 60 is written to, do not output a CR/LF or
               the printer will eject the page prior to step 5.

           6.  Process the graphics to the printer by calling PRINTPAGE.

           7.  Force the printer to print the page by calling TERMINATEPAGE.


         Note:  This unit assumes that the text to be printed on the HP
                Laser Jet Printer is at 10 CPI and 6 LPI.  If not, then
                rebuild this unit after altering values for constants
                CPI and LPI ( and please rename the Unit ).

                The original source is set up for landscape page orientation.
                A source for portrait page orientation ( file named
                HPLJ75p.PAS ) is available provides all of the above
                functions in that mode.

                Should a programmer have need for both portrait and landscape
                version of this Unit in the same program, the second Unit would
                require a unique (global) interface.



}



unit HPLJ;     { modified for dashed lines with neg (-) linewidth, Apr 89 }
               { modified for greater page coverage, Aug '90 }

interface

uses CRT, PRINTER;

{ Global procedure interface }


                                           { control procedures }
procedure InitBitMap;                         { reset/init bit map }
procedure ResetLaser;                         { reset printer }
procedure LandScapeMode;                      { set printer to landscape mode }
procedure PrintPage;                          { process grf data to printer }
procedure TerminatePage;                      { force printer output }

                                           { lowest level procedure to test }
                                                          { and set bit/dots }
procedure SetBit(col, row :integer; BitOnMode : boolean); { and is used by }
                                                          { of the following }
                                           { universal line draw routine }

procedure DoLine( X1, Y1, X2, Y2 : real;      { coordinates in inches }
                       LineWidth : integer;   { line width in dots }
                          Dspace : integer);  { dots between double lines }

                                            { cell center draw line }
procedure HchrLn(StartX, StartY, EndX, Width, Space : integer);  { procedures }
procedure VchrLn(StartX, StartY, EndY, Width, Space : integer);

                                            { cell boundry draw line }
procedure HbndLn(StartX, StartY, EndX, Width, Space : integer);  { procedures }
procedure VbndLn(StartX, StartY, EndY, Width, Space : integer);

                                             { draw box, cell and boundry }
procedure FrameChr(StartX, StartY, EndX, EndY : integer);
procedure FrameBnd(StartX, StartY, EndX, EndY : integer);

var   SetBitError : boolean;  { global - used by SETBIT procedure }


implementation      { note: all const, type and var are unit local }
{----------------------------------------------------------------------------}

const
          CPI        = 10;      { text printed at Characters Per Inch }
          LPI        = 6;       { text printed at Lines Per Inch }
          DPI        = 75;      { dots per inch }
          DPL        = DPI/LPI; { dots per line }
          DPC        = DPI/CPI; { dots per character }
          PageWidth  = 11;      { image inches of 11" wide page mode }
          PageLength = 8.5;     { image inches of 8 1/2" long page mode}
          Dot        = 1/DPI;   { dot size, Dot = 0.01333 inch @ 75 DPI }
          BitOn      = true;
          BitOff     = false;
      PageMaxRow     = trunc(PageLength * DPI) -1;   { 637.5 rows; 0 to 636 }
      PageMaxCol     = (PageWidth  * DPI) -1;          { 825 cols; 0 to 824 }
      BytesTall      = (trunc(DPI * PageLength) div 8) -1;   { 0 to 78 tall }

type
    PageLine  =  record                { StartByte and EndByte point to the }
                   StartByte  : byte;  { implied bytes.  If StartByte = $FF }
                   EndByte    : byte;  { then line contains all white data  }
                   data       : array[0..BytesTall] of byte;
                 end;     { record }


       pPline =  ^pageLine;
   PageBitMap =  array[0..PageMaxCol] of pPline;
         str8 =  string[8];


const
                     Esc = #27;
                      Cr = #13;
                      FF = #12;
     ResetPrinter : str8 = Esc + 'E';       { printer reset string }
        Landscape : str8 = Esc + '&l1O';    { put printer in LandScape mode }
        Terminate : str8 = Cr + FF;         { print page string }
              HMI : str8 = Esc + '&k1.6h';  { hard coded for 75 DPI }
              VMI : str8 = Esc + '&l.64C';  { hard coded for 75 DPI }
       Resolution : str8 = Esc + '*t75R';   { hard coded resolution }
    StartGraphics : str8 = Esc + '*r1A';
      EndGraphics : str8 = Esc + '*rB';

var
       PageRow : PageLine;
         Ppage : PageBitMap;                     { memory array on the heap }

{----------------------------------------------------------------------------}
procedure Init;
var i : integer;
begin
  fillchar(PageRow,sizeof(PageRow),$00);      { Init the page memory array }
  PageRow.StartByte := $FF;                   { set white data value }
  for i := 0 to PageMaxCol do
  begin
    new(Ppage[i]);
    Ppage[i]^ := PageRow;
  end;
end;
{----------------------------------------------------------------------------}
procedure InitBitMap;
var i : integer;
begin
  fillchar(PageRow,sizeof(PageRow),$00);  { Init the page mode memory array }
  PageRow.StartByte := $FF;
  for i := 0 to PageMaxCol do Ppage[i]^ := PageRow;
end;
{----------------------------------------------------------------------------}
procedure SetBit(col, row :integer; BitOnMode : boolean);
var    ByteNo, BitNO, Tmp, Byt : byte;
                        LineNo : integer;
                        BitLit : boolean;
begin
  SetBitError := true;                 { alter the state of a bit in the }
  Byt    := $01;                       { appropriate byte, if required }
  ByteNo := (row div 8);
  BitNo  := 7 - (row mod 8);   { **** Note **** the row and col arguments }
  LineNo := col;               {                are fliped in this Unit as }

  if (col > PageMaxCol) or (row > PageMaxRow) then exit;

  Tmp := Ppage[LineNo]^.data[ByteNo];        {  compared to the Portrait ver }
  if odd(Tmp shr BitNo) then BitLit := BitOn else BitLit := BitOff;

  if BitOnMode and not BitLit then Tmp := Tmp + (Byt shl BitNo)
   else
  if not BitOnMode and BitLit then Tmp := Tmp - (Byt shl BitNo);

  Ppage[LineNo]^.data[ByteNo] := Tmp;                  { put the byte back }

  if BitOnMode then                { set/reset flags for this line of data }
  begin
    if Ppage[LineNo]^.StartByte > ByteNo then
       Ppage[LineNo]^.StartByte := ByteNo;
    if Ppage[LineNo]^.EndByte < ByteNo then
       Ppage[LineNo]^.EndByte := ByteNo;
  end;
  SetBitError := false;
end;
{----------------------------------------------------------------------------}
{
  Note: if performance were an issue, this routine should be replaced for all
        line drawing activity where lines are Horizontal or Vertical.  The
        overhead in this procedure which allows it to draw from any point
        to any to other point could be avoided.
}



procedure DoLine( X1, Y1, X2, Y2 : real;      { coordinates in inches }
                       LineWidth : integer;   { line width in dots }
                          Dspace : integer);  { dots between double lines }

var   cnt, sX, sY, eX, eY, x, y,
      Xstep, Ystep, DeltaX, DeltaY,
      direction, linecnt, tmp, dash    : integer;
                     dashMode, BitMode : boolean;
begin

  dashMode := false;
  sX := trunc(X1/Dot);        { converts argument coordinates from inches to }
  sY := trunc(Y1/Dot);                 { Dot coordinates on the bit map page }
  eX := trunc(X2/Dot);
  eY := trunc(Y2/Dot);

  if sY > eY then                      { swap to insure tops down processing }
  begin
    tmp := sY;  sY  := eY;  eY  := tmp;
    tmp := sX;  sX  := eX;  eX  := tmp;
  end;

  if LineWidth < 0 then
  begin
    dashMode := true;
    LineWidth := abs(LineWidth);
  end;

  if LineWidth = 0 then                        { set to loop LineWidth times }
  begin                                     { if LineWidth = 0 turn off bits }
    BitMode := BitOff;
    linecnt := 1;
  end
    else
  begin                               { else set total line processing count }
    if Dspace > 0 then linecnt := (2 * LineWidth) + Dspace
      else linecnt := LineWidth;
    BitMode := BitOn;                             { and set turn on bit mode }
  end;
  cnt := 0;
  dash := 0;

  repeat
    inc(cnt);
    x := sX;  y := sY;   { <---------------------- from here to next flag is }
                                                    { a fairly std line draw }
    if sX > eX then Xstep := -1 else Xstep := 1;                   { routine }
    if sY > eY then Ystep := -1 else Ystep := 1;

    DeltaX := abs(eX - sX);
    DeltaY := abs(eY - sY);

    if DeltaX = 0 then direction := -1 else direction := 0;

    while not ((x = eX) and (y = eY)) do
    begin
      if DashMode and (dash >= 8) then
      begin
        if dash >= 12 then dash := 0;
      end
       else
      SetBit(x,y,BitMode);
      inc(dash);
      if direction < 0 then
      begin
        y := y + Ystep;
        direction := direction + DeltaX;
      end
        else
      begin
        x := x + Xstep;
        direction := direction - DeltaY;
      end;
    end;

    if DeltaX >= DeltaY then
    begin
      sY := sY + 1;
      eY := eY + 1;
    end
      else
    begin
      sX := sX + 1;
      eX := eX + 1;
    end;               { <------------------- flag end of std draw rountine }

                                               { manage space between lines }
    if (cnt + 1 > LineWidth) and (cnt + 1 <= (LineWidth + Dspace)) then
      BitMode := BitOff
       else
      BitMode := BitOn;

  until cnt = linecnt;
end;
{----------------------------------------------------------------------------}
procedure HchrLn(StartX, StartY, EndX, Width, Space : integer);
var X1, Y, X2 : real;
begin
  Y  := ((StartY  * DPL) - DPL/2) * Dot;
  X1 := ((StartX  * DPC) - DPC/2) * Dot;
  X2 := (((EndX   * DPC) - DPC/2) * Dot) + Dot;
  DoLine(X1,Y,X2,Y,Width,Space);
end;
{----------------------------------------------------------------------------}
procedure VchrLn(StartX, StartY, EndY, Width, Space  : integer);
var X, Y1, Y2 : real;
begin
  Y1 := ((StartY * DPL)  - DPL/2) * Dot;
  X  := ((StartX * DPC)  - DPC/2) * Dot;
  Y2 := ((EndY   * DPL)  - DPL/2) * Dot;
  DoLine(X,Y1,X,Y2,Width,Space);
end;
{----------------------------------------------------------------------------}
procedure HbndLn(StartX, StartY, EndX, Width, Space : integer);
var X1, Y, X2 : real;
begin
  Y  := ((StartY -1) * DPL  * Dot) - DOT;
  X1 := (StartX -1)  * DPC  * Dot;
  X2 := ((EndX-1)    * DPC  * Dot) + Dot;
  DoLine(X1,Y,X2,Y,Width,Space);
end;
{----------------------------------------------------------------------------}
procedure VbndLn(StartX, StartY, EndY, Width, Space : integer);
var X, Y1, Y2 : real;
begin
  Y1 := ((StartY - 1) * DPL  * Dot) - Dot;
  X  := (StartX -1)   * DPC  * Dot;
  Y2 := ((EndY - 1)   * DPL  * Dot) - Dot;
  DoLine(X,Y1,X,Y2,Width,Space);
end;
{----------------------------------------------------------------------------}
procedure FrameChr(StartX, StartY, EndX, EndY : integer);
begin
  HchrLn(StartX, StartY, EndX, 1, 0);
  HchrLn(StartX, EndY,   EndX, 1, 0);
  VchrLn(StartX, StartY, EndY, 1, 0);
  VchrLn(EndX,   StartY, EndY, 1, 0);
end;
{----------------------------------------------------------------------------}
procedure FrameBnd(StartX, StartY, EndX, EndY : integer);
begin
  HbndLn(StartX,  StartY, EndX + 1, 1, 0);
  HbndLn(StartX,  EndY,   EndX + 1, 1, 0);
  VbndLn(StartX,  StartY, EndY, 1, 0);
  VbndLn(EndX +1, StartY, EndY, 1, 0);
end;
{----------------------------------------------------------------------------}
procedure ResetLaser;
begin
  write(lst,ResetPrinter);
end;
{----------------------------------------------------------------------------}
procedure LandScapeMode;
begin
 write(lst,Landscape);    { put printer in LandScape mode }
end;
{----------------------------------------------------------------------------}
procedure TerminatePage;
begin
  write(lst,Terminate);
end;
{----------------------------------------------------------------------------}
procedure PrintPage;              { processes the bitmap data to the printer }
var  i, j, k : integer;    { note that RESET is called before processing any }
     EOFmark : boolean;                    { text or graphics to the printer }
     MarkCnt : integer;
     EOFloc  : array[1..PageMaxRow] of integer;

   procedure AltProcessing(col : integer);
   var i, j, k : integer;
   begin  { first process the col of data, deleting the $1A from the byte
            stream, insetring a null in its place. }

     write(lst,Esc + '&a',PageRow.StartByte * 8,'r',col,'C');
     write(lst,StartGraphics);
     write(lst,Esc,'*b',(PageRow.EndByte - PageRow.StartByte) + 1,'W');
     for j := PageRow.StartByte to PageRow.EndByte do
       if PageRow.data[j] = $1A then
         write(lst,chr($00))
           else
         write(lst,chr(PageRow.data[j]));
     write(lst,EndGraphics);

           { now, process those bytes that are $1A by positioning printer
             cursor one dot into the byte and outputing a $34 or $35 instead. }
     for i := 1 to MarkCnt do
     begin                         { shift one bit/dot into the byte location }
       write(lst,Esc + '&a',(EOFloc[i] * 8) + 1,'r',col,'C');
       write(lst,StartGraphics);
       write(lst,Esc,'*b',1,'W');
       if i < PageMaxRow then      { check hi bit of next byte and pick it up }
         if EOFloc[i] + 1 >= 128 then write(lst,chr($35))        { if present }
           else
            write(lst,chr($34));
       write(lst,EndGraphics);
     end;
   end; { AltProcessing }

begin
  write(lst,HMI);               { infrom printer which mode that the data is }
  write(lst,VMI);                                         { to being sent in }
  write(lst,Resolution);

  for i := 0 to PageMaxCol do         { loop through page bit map col by col }
  begin
    PageRow := Ppage[i]^;
    if PageRow.StartByte <> $FF then           { enter loop if line has data }
    begin
      EOFMark := false;
      MarkCnt := 0;
      for k := PageRow.StartByte to PageRow.EndByte do  { set up to identify }
        if PageRow.data[k] = $1A then                   { and process col in }
        begin                                           { an alternate manner}
          EOFmark := true;                            { if $1A chraracter(s) }
          inc(MarkCnt);                          { is/are in the data stream }
          EOFloc[MarkCnt] := k;
        end;
      if EOFmark then AltProcessing(i)
       else
      begin
        write(lst,Esc + '&a',PageRow.StartByte * 8,'r',i,'C');  { X,Y position }
                                                            { cursor on page }
        write(lst,StartGraphics);                        { inform printer that }
                                                         { grf. data follows }
                                                 { inform printer the number }
        write(lst,Esc,'*b',(PageRow.EndByte - PageRow.StartByte) + 1,'W');
                                                       { of bytes to be sent }
        for j := PageRow.StartByte to PageRow.EndByte do       { process bytes }
          write(lst,chr(PageRow.data[j]));                    { to the printer }
        write(lst,EndGraphics);            { inform printer "end" of this data }
      end; { std col/row process data to printer loop }

    end; { print this row data loop }
  end; { row select loop }
end;
{----------------------------------------------------------------------------}
begin
  Init;     { allocate memory and init it }
end.
{----------------------------------------------------------------------------}


















